Package: sensobol
Title: Computation of Variance-Based Sensitivity Indices
Version: 1.2.0
Authors@R: c(person(given = "Arnald", 
                    family = "Puy",
                    role = c("aut", "cre"),
                    email = "arnald.puy@pm.me", 
                    comment = c(ORCID = "https://orcid.org/0000-0001-9469-2156")), 
            person("Bertrand", "Ioos", 
                    role = "ctb", 
                    comment = "Author of included 'sensitivity' fragments"), 
            person("Gilles", "Pujol", 
                    role = "ctb", 
                    comment = "Author of included 'sensitivity' fragments"), 
            person("RStudio", 
                    role = "cph", 
                    comment = "Copyright holder of included 'sensitivity' fragments"))
Maintainer: Arnald Puy <arnald.puy@pm.me>
Description: It allows to rapidly compute, bootstrap and plot up to fourth-order Sobol'-based sensitivity indices using several state-of-the-art first and total-order estimators. Sobol' indices can be computed either for models that yield a scalar as a model output or for systems of differential equations. The package also provides a suit of benchmark tests functions and several options to obtain publication-ready figures of the model output uncertainty and sensitivity-related analysis. An overview of the package can be found in Puy et al. (2022) <doi:10.18637/jss.v102.i05>. 
License: GPL-3
Encoding: UTF-8
Imports: boot (>= 1.3.20), data.table (>= 1.12.0), ggplot2 (>= 3.1.0),
        lhs (>= 1.0.2), magrittr (>= 1.5), matrixStats (>= 0.54.0),
        randtoolbox (>= 1.17.1), deSolve (>= 1.27.1), Rdpack (>=
        2.1.2), Rfast (>= 2.0.1), rlang (>= 0.3.1), scales (>= 1.0.0),
        stats, stringr (>= 1.4.0), utils, Rcpp
RdMacros: Rdpack
Depends: R (>= 3.5.0)
RoxygenNote: 7.3.3
Suggests: knitr, rmarkdown, testthat (>= 2.1.0), covr
VignetteBuilder: knitr
URL: https://github.com/arnaldpuy/sensobol
BugReports: https://github.com/arnaldpuy/sensobol/issues
LinkingTo: Rfast, Rcpp, RcppArmadillo,
NeedsCompilation: yes
Packaged: 2026-07-17 07:00:37 UTC; arnaldpuy
Author: Arnald Puy [aut, cre] (ORCID: <https://orcid.org/0000-0001-9469-2156>),
  Bertrand Ioos [ctb] (Author of included 'sensitivity' fragments),
  Gilles Pujol [ctb] (Author of included 'sensitivity' fragments),
  RStudio [cph] (Copyright holder of included 'sensitivity' fragments)
Repository: CRAN
Date/Publication: 2026-07-17 07:30:02 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-07-17 07:47:15 UTC; unix
Archs: sensobol.so.dSYM
