# Generated by roxygen2: do not edit by hand

S3method(plot,sensobol)
S3method(print,sensobol)
S3method(print,vars)
export(bratley1988_Fun)
export(bratley1992_Fun)
export(discrepancy_ersatz)
export(ishigami_Fun)
export(load_packages)
export(metafunction)
export(oakley_Fun)
export(plot_multiscatter)
export(plot_scatter)
export(plot_uncertainty)
export(sobol_Fun)
export(sobol_convergence)
export(sobol_dummy)
export(sobol_indices)
export(sobol_matrices)
export(sobol_ode)
export(vars_matrices)
export(vars_to)
import(ggplot2)
importFrom(Rcpp,sourceCpp)
importFrom(Rdpack,reprompt)
importFrom(data.table,.N)
importFrom(data.table,.SD)
importFrom(magrittr,"%>%")
importFrom(rlang,":=")
importFrom(stats,var)
useDynLib(sensobol, .registration = TRUE)
